geodist = function(long1, lat1, long2, lat2)
{
  if (!is.numeric(long1))
  {
    long1 = gsub(",", ".", long1)
    long1 = as.numeric(long1)
  }
  if (!is.numeric(lat1))
  {
    long1 = gsub(",", ".", long1)
    long1 = as.numeric(long1)
  }
  
  if (!is.numeric(long2))
  {
    long2 = gsub(",", ".", long2)
    long2 = as.numeric(long2)
  }
  if (!is.numeric(lat2))
  {
    lat2 = gsub(",", ".", lat2)
    lat2 = as.numeric(lat2)
  }
  
  temp1 = matrix(c(long1, lat1), ncol=2)
  temp2 = matrix(c(long2, lat2), ncol=2)
  D = geosphere::distHaversine(temp1, temp2)
  D = matrix(D)
  
  return (D)
}

time_dif_data = function(timestop_busstop, bstop_data)
{
  time_dif_data = NULL
  time_dif_dayhour = NULL
  time_dif_data_index = 0
  
  if (length(timestop_busstop[,2]) == 0)
  {
    return (time_dif_data)
  }
  
  origin = bstop_data[1]
  
  i = 1;
  while(timestop_busstop[i,2] == origin & i < length(timestop_busstop[,2]))
  {
    i = i + 1
  }
  
  time_dif_data = NULL
  time_dif_dayhour = NULL
  time_dif_data_index = 0
  
  if (i == length(timestop_busstop[,2]))
  {
    return (time_dif_data)
  }
  
  destination = timestop_busstop[i,2]
  
  
  
  for (i in 1:(length(timestop_busstop[,1]) - 1))
  {
    if(timestop_busstop[i,2] == origin && timestop_busstop[i+1, 2] == destination)
    {
      time_dif_data_index = time_dif_data_index + 1
      time_dif_data[time_dif_data_index] = difftime(strptime(timestop_busstop[i+1,1], "%Y-%m-%d %H:%M:%S" ), strptime(timestop_busstop[i,1], "%Y-%m-%d %H:%M:%S" ), units = "secs")
      time_dif_dayhour[time_dif_data_index] =timestop_busstop[i,1]
    }
  }
  
  #time_dif_data = matrix(time_dif_data, ncol = 2, byrow = TRUE)
  time_dif_data = data.frame(time_dif_data, time_dif_dayhour)
  return (time_dif_data)
}

time2TCS = function(time_stops)
{
  TCS_temp = time_stops
  ref_time = getmode(sort(time_stops[,1])[1:as.integer((length(time_stops))/4)])
  TCS_temp[,1] = 1 - ref_time/time_stops[,1]
  TCS_temp[,1] = TCS_temp[,1]*(TCS_temp[,1] >0)
  names(TCS_temp)[1] = "TCS"
  
  return (TCS_temp)
}

remove_outliers <- function(x, na.rm = TRUE, ...) {
  qnt <- quantile(x[,1], probs=c(.25, .75), na.rm = na.rm)
  H <- 1.5 * IQR(x[,1], na.rm = na.rm)
  y <- x[(x[,1] > qnt[1] - H & x[,1] < qnt[2] + H),]
  return(y)
}

getmode <- function(v) {
  uniqv <- unique(v)
  uniqv[which.max(tabulate(match(v, uniqv)))]
}

plot_hist = function(data)
{
  h <- hist(unlist(data))
  h$xname = names(data)
  plot(h)
}

plot_boxplot_TCS_time = function(data)
{
  par(mar=c(5.1, 4.1, 4.1, 2.1), mgp=c(3, 1, 0), las=0)
  boxplot(data[[1]][,1] ~ data[[1]][,2], ylim = c(0,1), ylab = "TCS", cex.axis =0.7,
          main = paste("TCS em função da hora\nLinha: ", ID_LINHA, "; Ônibus:", 
                       paste(circulino_bus_ids, collapse=", "),"\n", names(data), sep = ""), 
          cex.main = 0.7)
}